@@ -1,8 +1,8 @@
 # proc: Simple interface to Linux process information.
 #
 # Author: Peter Odding <peter@peterodding.com>
-# Last Change: April 21, 2016
-# URL: https://proc.readthedocs.org
+# Last Change: April 22, 2016
+# URL: https://proc.readthedocs.io
 
 """
 The :mod:`proc.notify` module implements a headless notify-send_ program.
@@ -22,8 +22,10 @@ This module builds on top of the :mod:`proc.core` module as a trivial (but
 already useful :-) example of how the `proc` package can be used to search
 through the environments of all available processes. It looks for the variables
 in :attr:`REQUIRED_VARIABLES` in the environments of all available processes
-and uses the values it finds to run the notify-send program. Given super-user
-privileges this should work fine out of the box on any Linux system.
+and uses the values it finds to run the notify-send program. It's available on
+the command line as ``notify-send-headless`` (which accepts the same arguments
+as ``notify-send``). Given super-user privileges this should work fine out of
+the box on any Linux system.
 
 .. _cron jobs: http://unix.stackexchange.com/q/111188
 .. _notify-send: http://manpages.debian.org/cgi-bin/man.cgi?query=notify-send
@@ -102,9 +104,9 @@ def find_graphical_context():
         if environment:
             hashable_environment = tuple(sorted(environment.items()))
             matches[(process.user_ids.real, hashable_environment)] += 1
-    if matches:
+    ordered = sorted((counter, key) for key, counter in matches.items())
+    if ordered:
         # Pick the most popular graphical session.
-        ordered = sorted((counter, key) for key, counter in matches.items())
         counter, key = ordered[-1]
         uid, environment = key
         # Apply the user ID to the context?
