@@ -1,8 +1,8 @@
 # Automated tests for the `proc' package.
 #
 # Author: Peter Odding <peter@peterodding.com>
-# Last Change: April 21, 2016
-# URL: https://proc.readthedocs.org
+# Last Change: June 24, 2017
+# URL: https://proc.readthedocs.io
 
 """Test suite for the `proc` package."""
 
@@ -15,51 +15,32 @@ import os
 import random
 import subprocess
 import time
-import unittest
 
 from pprint import pformat
 
 # External dependencies.
-import coloredlogs
 from executor import ExternalCommand, which
 from executor.contexts import AbstractContext
 from humanfriendly import parse_size, Timer
 from humanfriendly.compat import basestring
+from humanfriendly.testing import CustomSearchPath, MockedProgram, TestCase
 
 # Modules included in our package.
 from proc.apache import find_apache_memory_usage, StatsList
 from proc.core import Process, find_processes, gid_to_name, num_race_conditions, uid_to_name
-from proc.cron import cron_graceful, ensure_root_privileges, wait_for_processes
-from proc.notify import find_graphical_context
+from proc.cron import ADDITIONS_SCRIPT_NAME, cron_graceful, ensure_root_privileges, run_additions, wait_for_processes
+from proc.gpg import get_gpg_variables, with_gpg_agent
+from proc.notify import REQUIRED_VARIABLES, find_graphical_context, notify_desktop
 from proc.tree import get_process_tree
+from proc.unix import UnixProcess
 
 # Initialize a logger.
 logger = logging.getLogger(__name__)
 
 
-def setUpModule():
-    """
-    Prepare the test suite.
+class ProcTestCase(TestCase):
 
-    Sets up verbose logging to the terminal. When a test fails the logging
-    output can help to perform a post-mortem analysis of the failure in
-    question (even when its hard to reproduce locally). This is especially
-    useful when debugging remote test failures, whether they happened on Travis
-    CI or a user's local system.
-    """
-    # Initialize verbose logging to the terminal.
-    coloredlogs.install()
-
-
-class ProcTestCase(unittest.TestCase):
-
-    """
-    :mod:`unittest` compatible container for the test suite of `proc`.
-    """
-
-    def setUp(self):
-        """Reset the logging level before every test runs."""
-        coloredlogs.set_level(logging.DEBUG)
+    """:mod:`unittest` compatible container for the test suite of `proc`."""
 
     def test_uid_to_name(self):
         """Make sure :func:`uid_to_name()` never raises exceptions."""
@@ -121,7 +102,7 @@ class ProcTestCase(unittest.TestCase):
         # Test some basic assumptions about the result of find_processes().
         processes = dict((p.pid, p) for p in find_processes())
         assert 1 in processes, "init process not found in output of find_processes()!"
-        assert processes[1].comm == 'init', "init isn't called init?!"
+        assert processes[1].comm in ('init', 'systemd')
         assert os.getpid() in processes, "Current process not found in output of find_processes()!"
 
     def test_is_alive(self):
@@ -146,19 +127,49 @@ class ProcTestCase(unittest.TestCase):
             sleep_proc = Process.from_pid(sleep_cmd.pid)
             assert sleep_proc.environ['unique_value'] == unique_value
 
+    def test_get_gpg_variables(self):
+        """Test that searching for gpg-agents works."""
+        if not which('gpg-agent'):
+            return self.skipTest("The gpg-agent program is not installed!")
+        variables = get_gpg_variables()
+        assert variables['GPG_AGENT_INFO']
+
+    def test_with_gpg_agent(self):
+        """Test that ``with-gpg-agent`` works."""
+        if not which('gpg-agent'):
+            return self.skipTest("The gpg-agent program is not installed!")
+        # Test that `with-gpg-agent -h' / `with-gpg-agent --help' works.
+        self.assertRaises(SystemExit, with_gpg_agent, ['-h'])
+        self.assertRaises(SystemExit, with_gpg_agent, ['--help'])
+        # Test that invalid command line options raise an error.
+        self.assertRaises(SystemExit, with_gpg_agent, ['--whatever'])
+        # Test that command line options for verbosity control are accepted
+        # and that the with-gpg-agent program runs successfully.
+        with_gpg_agent(['-v', '--verbose', '-q', '--quiet', 'true'])
+
     def test_find_graphical_context(self):
         """Test that :func:`proc.notify.find_graphical_context()` works."""
         context = find_graphical_context()
         assert isinstance(context, AbstractContext)
         assert context.execute('true', check=False)
 
+    def test_notify_desktop(self):
+        """Test that :func:`proc.notify.notify_desktop()` works."""
+        env = dict((name, 'value') for name in REQUIRED_VARIABLES)
+        with ExternalCommand('sleep 60', environment=env):
+            with MockedProgram('notify-send'):
+                notify_desktop(
+                    summary="Headless notifications",
+                    body="They actually work! (this message brought to you by the 'proc' test suite :-)",
+                    urgency='low',
+                )
+
     def test_exe_path_fallback(self):
         """Test the fall back method of :attr:`proc.core.Process.exe_path`."""
         candidates = [p for p in find_processes() if p.exe_path and not p.exe]
         logger.debug("Candidates for Process.exe_path fall back test:\n%s", pformat(candidates))
         if not candidates:
-            logger.debug("Skipping test: No processes available on which Process.exe_path fall back can be tested!")
-            return
+            return self.skipTest("No processes available on which Process.exe_path fall back can be tested!")
         assert executable(candidates[0].exe_path), \
             "Fall back method of Process.exe_path reported invalid executable pathname!"
 
@@ -167,13 +178,11 @@ class ProcTestCase(unittest.TestCase):
         if os.getuid() == 0:
             # Given root privileges all /proc/[pid]/exe symbolic links can be
             # successfully resolved so we can't test the fall back method.
-            logger.debug("Skipping test: Fall back method of Process.exe_name is useless with root privileges!")
-            return
+            return self.skipTest("Fall back method of Process.exe_name is useless with root privileges!")
         candidates = [p for p in find_processes() if p.exe_name and not p.exe_path]
         logger.debug("Candidates for Process.exe_name fall back test:\n %s", pformat(candidates))
         if not candidates:
-            logger.debug("Skipping test: No processes available on which Process.exe_name fall back can be tested!")
-            return
+            return self.skipTest("No processes available on which Process.exe_name fall back can be tested!")
         assert any(which(p.exe_name) for p in candidates), \
             "Fall back method of Process.exe_name reported executable base name not available on $PATH?!"
 
@@ -249,6 +258,22 @@ class ProcTestCase(unittest.TestCase):
         # that a dry run of cron-graceful runs successfully.
         cron_graceful(['-q', '--quiet', '-v', '--verbose', '-n', '--dry-run'])
 
+    def test_cron_graceful_additions(self):
+        """Test :func:`proc.cron.run_additions()`."""
+        # If no program with the name cron-graceful-additions is available
+        # on the $PATH we expect this to be handled ... gracefully :-).
+        with CustomSearchPath(isolated=True):
+            run_additions()
+        # If a program with the name cron-graceful-additions is available
+        # on the $PATH we expect it to be executed successfully.
+        with MockedProgram(ADDITIONS_SCRIPT_NAME):
+            run_additions()
+        # If the cron-graceful-additions program fails we also expect this
+        # to be handled gracefully (i.e. a message is logged but the error
+        # isn't propagated).
+        with MockedProgram(ADDITIONS_SCRIPT_NAME, returncode=1):
+            run_additions()
+
     def test_race_conditions(self, timeout=60):
         """
         Test the handling of race conditions in :mod:`proc.core`.
@@ -330,8 +355,7 @@ class ProcTestCase(unittest.TestCase):
     def test_apache_worker_monitoring(self):
         """Test the :mod:`proc.apache` module."""
         if not os.path.exists('/etc/apache2/sites-enabled/proc-test-vhost'):
-            logger.debug("Skipping test: Apache worker monitoring test disabled except on Travis CI!")
-            return
+            return self.skipTest("Apache worker monitoring test disabled except on Travis CI!")
         worker_rss, wsgi_rss = find_apache_memory_usage()
         # Make sure some regular Apache workers were identified.
         assert len(worker_rss) > 0, "No regular Apache workers found?!"
@@ -340,12 +364,79 @@ class ProcTestCase(unittest.TestCase):
         # identification of WSGI workers requires root privileges, so
         # without that there's no point in running the test (we know it
         # will fail).
-        if not os.getuid() == 0:
-            logger.debug("Skipping test: Apache WSGI worker monitoring test requires root privileges!")
-            return
+        if os.getuid() != 0:
+            return self.skipTest("Apache WSGI worker monitoring test requires root privileges!")
         assert 'proc-test' in wsgi_rss
         assert wsgi_rss['proc-test'].average > 0
 
+    def test_graceful_termination(self):
+        """Test graceful process termination."""
+        self.check_process_termination(method='terminate')
+
+    def test_forceful_termination(self):
+        """Test forceful process termination."""
+        self.check_process_termination(method='kill')
+
+    def check_process_termination(self, method):
+        """Helper method for process termination tests."""
+        timer = Timer()
+        # We use Executor to launch an external process.
+        with ExternalCommand('sleep', '60', check=False) as cmd:
+            # Verify that proc.unix.UnixProcess.is_running (which is normally
+            # overridden by proc.core.Process.is_running) works as expected,
+            # even though this property isn't actively used in the `proc'
+            # package because we want to deal with not-yet-reclaimed
+            # processes and zombie processes which is very much a Linux
+            # specific thing (hence the override).
+            unix_process = UnixProcess(pid=cmd.pid)
+            assert unix_process.is_running, "UnixProcess.is_running is broken!"
+            # We don't use Executor to control the process, instead we take the
+            # process ID and use it to create a Process object that doesn't
+            # know about Python's subprocess module.
+            linux_process = Process.from_pid(cmd.pid)
+            # We terminate the process using a positive but very low timeout so
+            # that all of the code involved gets a chance to run, but without
+            # significantly slowing down the test suite.
+            getattr(linux_process, method)(timeout=0.1)
+            # Now we can verify our assertions.
+            assert not linux_process.is_running, "Child still running despite graceful termination request!"
+            assert timer.elapsed_time < 10, "It look too long to terminate the child!"
+            # Now comes a hairy bit of Linux implementation details that most
+            # people can thankfully ignore (blissful ignorance :-). Parent
+            # processes are responsible for reclaiming child processes and
+            # until this happens the /proc/[pid] entry remains, which means
+            # the `kill -0' trick used by UnixProcess to detect running
+            # processes doesn't work as expected. Basically this means we
+            # _must_ make sure that waitpid() is called before we can expect
+            # UnixProcess.is_running to behave as expected.
+            cmd.wait()
+            # Now that we've called waitpid() things should work as expected.
+            assert not unix_process.is_running, "UnixProcess.is_running is broken!"
+
+    def test_is_running(self):
+        """Test that UnixProcess.is_running gracefully handles processes it doesn't have permission to query."""
+        init = UnixProcess(pid=1)
+        assert init.is_running
+
+    def test_suspend_and_resume_signals(self):
+        """Test the sending of ``SIGSTOP``, ``SIGCONT`` and ``SIGTERM`` signals."""
+        # Spawn a child that will live for a minute.
+        with ExternalCommand('sleep', '60', check=False) as cmd:
+            process = Process.from_pid(cmd.pid)
+            # Suspend the execution of the child process using SIGSTOP.
+            process.suspend()
+            # Test that the child process doesn't respond to SIGTERM once suspended.
+            process.terminate(wait=False)
+            assert process.is_running, "Child responded to signal even though it was suspended?!"
+            # Resume the execution of the child process using SIGCONT.
+            process.resume()
+            # Test that the child process responds to signals again after
+            # having been resumed, but give it a moment to terminate
+            # (significantly less time then the process is normally expected
+            # to run, otherwise there's no point in the test below).
+            process.kill(wait=True, timeout=5)
+            assert not process.is_running, "Child didn't respond to signal even though it was resumed?!"
+
 
 def executable(pathname):
     """Check whether a pathname is executable."""
